//+------------------------------------------------------------------+
//|                                Darma System Indicator (beta).mq4 |
//|                                                tonyc2a@yahoo.com |
//|                                                                  |
//+------------------------------------------------------------------+
/*README
   This indicator is supposed to be a conversion of Darma's set of indicators from MT3 to MT4.
   This conversion is not exactly the same though. In the MT3 version, the indicators were split up between 4 separate custom indicator files. 
   This MT4 version has all the logic combined into one file. Due to this consolidation, some of the logic did not translate exactly as intended. 
   Also, I only did a direct code translation. Prior to looking at the code in the MT3 files, I had no understanding of the logic behind the indicator files. 
   If someone could provide the logic in human language, I might be able to make this indicator behave more properly as well as be a little more programmatically elegant. 
   Test this indicator out according to Darma's original rules and see how it does for you.
   
   Darma's Rules:
   BUY:
   1. When red stop dot (above price) becoming a blue stop dot (below price), the mood is ready for a buy. 
   3. Wait for the bar becoming blue. Sometime it happens in the same time, but sometime need to wait.  
   4. After you have a blue bar, Wait that blue bar closed completely. 
   5. Then place order to buy at 10-15 pips above the high of that blue bar. 

   SELL:
   1. When bule stop dot (below price) becoming a red stop dot (above price), the mood is ready for a sell.  
   3. Wait for the bar becoming red. Sometime it happens in the same time, but sometime need to wait.  
   4. After you have a red bar, Wait that red bar closed completely. 
   5. Then place order to sell at 10-15 pips below the low of that red bar. 
   
   PS. This code is very ugly and inelegant. I would like to fix this by getting system rules in english.
   
   - tonyc2a@yahoo.com 
*/
#property copyright "tonyc2a@yahoo.com"
#property link      ""

#property indicator_chart_window
#property indicator_buffers    4
#property indicator_color1     Lime
#property indicator_color2     Red
#property indicator_color3     Lime
#property indicator_color4     Red
#property indicator_width1     2
#property indicator_width2     2
#property indicator_style3     STYLE_DOT
#property indicator_width3     1
#property indicator_style4     STYLE_DOT
#property indicator_width4     1

//---- input parameters
extern string  TimeFrame        = "Current time frame";
extern bool    Extremes         = false;
extern int     WingDings_Symbol = 108;
extern bool    ArrowsOnFirstBar = true;
extern bool    alertsOn         = true;
extern bool    alertsOnCurrent  = true;
extern bool    alertsMessage    = true;
extern bool    alertsSound      = false;
extern bool    alertsNotify     = true;
extern bool    alertsEmail      = true;
extern string  soundFile        = "alert2.wav"; 

//---- buffers
double Buffer1[];
double Buffer2[];
double Buffer3[];
double Buffer4[];
double trend[];

string indicatorFileName;
bool   returnBars;
int    timeFrame;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
//
//

int init()
{
   IndicatorBuffers(5);
   SetIndexBuffer(0,Buffer1); SetIndexStyle(0,DRAW_HISTOGRAM); SetIndexLabel(0,"Blue Bar");
   SetIndexBuffer(1,Buffer2); SetIndexStyle(1,DRAW_HISTOGRAM); SetIndexLabel(1,"Red Bar");
   SetIndexBuffer(2,Buffer3); SetIndexStyle(2,DRAW_ARROW); SetIndexArrow(2,WingDings_Symbol); SetIndexLabel(2,"Blue Dot");
   SetIndexBuffer(3,Buffer4); SetIndexStyle(3,DRAW_ARROW); SetIndexArrow(3,WingDings_Symbol); SetIndexLabel(3,"Red Dot"); 
   SetIndexBuffer(4,trend);
   
   indicatorFileName = WindowExpertName();
   returnBars        = TimeFrame == "returnBars";     if (returnBars)     return(0);
   timeFrame         = stringToTimeFrame(TimeFrame);

//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custor indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
  {
//---- TODO: add your code here
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
  {
    int counted_bars=IndicatorCounted();
    int extremes=0;
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
           int limit=MathMin(Bars-counted_bars,Bars-1);
           if (returnBars) { Buffer1[0] = limit+1; return(0); }
   
            if (timeFrame!=Period())
            {
               int shift = -1; if (ArrowsOnFirstBar) shift=1;
               limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
               for (int i=limit; i>=0; i--)
               {
                   int y = iBarShift(NULL,timeFrame,Time[i]);   
                   int x = iBarShift(NULL,timeFrame,Time[i+shift]);  
                   if (x!=y)
                   {           
                     Buffer1[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",Extremes,0,y);
                     Buffer2[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",Extremes,1,y);
                     Buffer3[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",Extremes,2,y);
                     Buffer4[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",Extremes,3,y);
                     trend[i]   = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",Extremes,4,y);
                   }
               }
               manageAlerts();
               return(0);
            }
      
      for(i=limit; i>=0; i--)
      {
         if(!Extremes)
         {
            double SsMax = High[iHighest(NULL,0,MODE_OPEN,3,i)];
            double SsMin =  Low[iLowest(NULL, 0,MODE_OPEN,3,i)];
         }
         else
         {
            SsMax = Open[iHighest(NULL,0,MODE_OPEN,3,i)];
            SsMin = Open[iLowest(NULL, 0,MODE_OPEN,3,i)];
         }
         double smin = SsMin+(SsMax-SsMin)/100;
	      double smax = SsMax-(SsMax-SsMin)/100;      
         double pivothi=((2*High[i])+Low[i]+(2*Close[i]))/5;
         double pivotlo=(High[i]+(2*Low[i])+(2*Close[i]))/5;
         Buffer1[i] = EMPTY_VALUE;
         Buffer2[i] = EMPTY_VALUE;
         trend[i] = 0;
         if(pivotlo>smin && pivothi>smax) trend[i] = 1;
         if(pivothi<smax && pivotlo<smin) trend[i] =-1;
      
         if (trend[i]== 1)
         {
            Buffer1[i]=High[i];
            Buffer2[i]=Low[i];
         }
         if (trend[i]==-1)
         {
            Buffer1[i]=Low[i];
            Buffer2[i]=High[i];
         }
            
         double sig=iSAR(NULL,0,0.5,1,i);
         Buffer3[i] = EMPTY_VALUE;
         Buffer4[i] = EMPTY_VALUE;
         if(sig<Low[i])  Buffer3[i]=sig;
         if(sig>High[i]) Buffer4[i]=sig;
         
         
         
   }
manageAlerts();
return(0);
}
//+------------------------------------------------------------------+

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
//
//
//
//
//

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; whichBar = iBarShift(NULL,0,iTime(NULL,timeFrame,whichBar)); 
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar] == 1) doAlert(whichBar,"Buy");
         if (trend[whichBar] ==-1) doAlert(whichBar,"Sell");
      }         
   }
}   

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[forBar]) {
          previousAlert  = doWhat;
          previousTime   = Time[forBar];

          //
          //
          //
          //
          //

           message =  StringConcatenate(Symbol()," ",timeFrameToString(timeFrame)," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," Darma System ",doWhat);
             if (alertsMessage) Alert(message);
             if (alertsNotify)  SendNotification(StringConcatenate(Symbol(), Period() ," Darma System " +" "+message));
             if (alertsEmail)   SendMail(StringConcatenate(Symbol(), Period(), " Darma System "),message);
             if (alertsSound)   PlaySound(soundFile);
      }
}

